@@ -68,6 +68,14 @@ struct lttng_channel {
 	char padding[LTTNG_CHANNEL_PADDING1];
 };
 
+/*
+ */
+extern struct lttng_channel *lttng_channel_create(struct lttng_domain *domain);
+
+/*
+ */
+extern void lttng_channel_destroy(struct lttng_channel *channel);
+
 /*
  * List the channel(s) of a session.
  *
@@ -124,6 +132,18 @@ extern int lttng_channel_get_discarded_event_count(struct lttng_channel *chan,
 extern int lttng_channel_get_lost_packet_count(struct lttng_channel *chan,
 		uint64_t *lost_packets);
 
+extern int lttng_channel_get_monitor_timer_interval(struct lttng_channel *chan,
+		uint64_t *monitor_timer_interval);
+
+extern int lttng_channel_set_monitor_timer_interval(struct lttng_channel *chan,
+		uint64_t monitor_timer_interval);
+
+extern int lttng_channel_get_blocking_timeout(struct lttng_channel *chan,
+		int64_t *blocking_timeout);
+
+extern int lttng_channel_set_blocking_timeout(struct lttng_channel *chan,
+		int64_t blocking_timeout);
+
 #ifdef __cplusplus
 }
 #endif
