@@ -21,12 +21,14 @@
 #include <lttng/condition/evaluation.h>
 #include <common/macros.h>
 #include <common/buffer-view.h>
+#include <common/dynamic-buffer.h>
 #include <stdbool.h>
 #include <sys/types.h>
 
 typedef void (*evaluation_destroy_cb)(struct lttng_evaluation *evaluation);
-typedef ssize_t (*evaluation_serialize_cb)(struct lttng_evaluation *evaluation,
-		char *buf);
+typedef int (*evaluation_serialize_cb)(
+		const struct lttng_evaluation *evaluation,
+		struct lttng_dynamic_buffer *buf);
 
 struct lttng_evaluation_comm {
 	/* enum lttng_condition_type type */
@@ -40,12 +42,16 @@ struct lttng_evaluation {
 	evaluation_destroy_cb destroy;
 };
 
+LTTNG_HIDDEN
+void lttng_evaluation_init(struct lttng_evaluation *evaluation,
+		enum lttng_condition_type type);
+
 LTTNG_HIDDEN
 ssize_t lttng_evaluation_create_from_buffer(const struct lttng_buffer_view *view,
 		struct lttng_evaluation **evaluation);
 
 LTTNG_HIDDEN
-ssize_t lttng_evaluation_serialize(struct lttng_evaluation *evaluation,
-		char *buf);
+int lttng_evaluation_serialize(const struct lttng_evaluation *evaluation,
+		struct lttng_dynamic_buffer *buf);
 
 #endif /* LTTNG_EVALUATION_INTERNAL_H */
