@@ -1,5 +1,5 @@
-#ifndef RELAYD_CMD_2_1_H
-#define RELAYD_CMD_2_1_H
+#ifndef RELAYD_CMD_2_4_H
+#define RELAYD_CMD_2_4_H
 
 /*
  * Copyright (C) 2013 - Julien Desfossez <jdesfossez@efficios.com>
@@ -23,7 +23,8 @@
 #include "lttng-relayd.h"
 #include <common/buffer-view.h>
 
-int cmd_recv_stream_2_1(const struct lttng_buffer_view *payload,
-		char **path_name, char **channel_name);
+int cmd_create_session_2_4(const struct lttng_buffer_view *payload,
+		char *session_name, char *hostname,
+		uint32_t *live_timer, bool *snapshot);
 
-#endif /* RELAYD_CMD_2_1_H */
+#endif /* RELAYD_CMD_2_4_H */
