@@ -28,7 +28,8 @@
 #include "channel.h"
 #include "lttng-sessiond.h"
 #include "kernel.h"
-#include "ust-ctl.h"
+#include "lttng-ust-ctl.h"
+#include "lttng-ust-error.h"
 #include "utils.h"
 #include "ust-app.h"
 #include "agent.h"
