@@ -23,7 +23,7 @@
 struct agent;
 
 int event_kernel_disable_event(struct ltt_kernel_channel *kchan,
-		char *event_name, enum lttng_event_type event_type);
+		const char *event_name, enum lttng_event_type event_type);
 
 int event_kernel_enable_event(struct ltt_kernel_channel *kchan,
 		struct lttng_event *event, char *filter_expression,
@@ -36,7 +36,7 @@ int event_ust_enable_tracepoint(struct ltt_ust_session *usess,
 		struct lttng_event_exclusion *exclusion,
 		bool internal_event);
 int event_ust_disable_tracepoint(struct ltt_ust_session *usess,
-		struct ltt_ust_channel *uchan, char *event_name);
+		struct ltt_ust_channel *uchan, const char *event_name);
 
 int event_ust_disable_all_tracepoints(struct ltt_ust_session *usess,
 		struct ltt_ust_channel *uchan);
@@ -49,7 +49,7 @@ int event_agent_enable_all(struct ltt_ust_session *usess, struct agent *agt,
 		char *filter_expression);
 
 int event_agent_disable(struct ltt_ust_session *usess, struct agent *agt,
-		char *event_name);
+		const char *event_name);
 int event_agent_disable_all(struct ltt_ust_session *usess, struct agent *agt);
 
 const char *event_get_default_agent_ust_name(enum lttng_domain_type domain);
