@@ -62,7 +62,7 @@
 #include "kernel.h"
 #include "kernel-consumer.h"
 #include "shm.h"
-#include "ust-ctl.h"
+#include "lttng-ust-ctl.h"
 #include "ust-consumer.h"
 #include "utils.h"
 #include "fd-limit.h"
