@@ -123,13 +123,13 @@ enum lttng_error_code notification_thread_command_add_channel(
 	init_notification_thread_command(&cmd);
 
 	cmd.type = NOTIFICATION_COMMAND_TYPE_ADD_CHANNEL;
-	cmd.parameters.add_channel.session_name = session_name;
-	cmd.parameters.add_channel.uid = uid;
-	cmd.parameters.add_channel.gid = gid;
-	cmd.parameters.add_channel.channel_name = channel_name;
-	cmd.parameters.add_channel.key.key = key;
-	cmd.parameters.add_channel.key.domain = domain;
-	cmd.parameters.add_channel.capacity = capacity;
+	cmd.parameters.add_channel.session.name = session_name;
+	cmd.parameters.add_channel.session.uid = uid;
+	cmd.parameters.add_channel.session.gid = gid;
+	cmd.parameters.add_channel.channel.name = channel_name;
+	cmd.parameters.add_channel.channel.key = key;
+	cmd.parameters.add_channel.channel.domain = domain;
+	cmd.parameters.add_channel.channel.capacity = capacity;
 
 	ret = run_command_wait(handle, &cmd);
 	if (ret) {
@@ -165,6 +165,64 @@ end:
 	return ret_code;
 }
 
+enum lttng_error_code notification_thread_command_session_rotation_ongoing(
+		struct notification_thread_handle *handle,
+		const char *session_name, uid_t uid, gid_t gid,
+		uint64_t trace_archive_chunk_id)
+{
+	int ret;
+	enum lttng_error_code ret_code;
+	struct notification_thread_command cmd;
+
+	init_notification_thread_command(&cmd);
+
+	cmd.type = NOTIFICATION_COMMAND_TYPE_SESSION_ROTATION_ONGOING;
+	cmd.parameters.session_rotation.session_name = session_name;
+	cmd.parameters.session_rotation.uid = uid;
+	cmd.parameters.session_rotation.gid = gid;
+	cmd.parameters.session_rotation.trace_archive_chunk_id =
+			trace_archive_chunk_id;
+
+	ret = run_command_wait(handle, &cmd);
+	if (ret) {
+		ret_code = LTTNG_ERR_UNK;
+		goto end;
+	}
+	ret_code = cmd.reply_code;
+end:
+	return ret_code;
+}
+
+enum lttng_error_code notification_thread_command_session_rotation_completed(
+		struct notification_thread_handle *handle,
+		const char *session_name, uid_t uid, gid_t gid,
+		uint64_t trace_archive_chunk_id,
+		struct lttng_trace_archive_location *location)
+{
+	int ret;
+	enum lttng_error_code ret_code;
+	struct notification_thread_command cmd;
+
+	init_notification_thread_command(&cmd);
+
+	cmd.type = NOTIFICATION_COMMAND_TYPE_SESSION_ROTATION_COMPLETED;
+	cmd.parameters.session_rotation.session_name = session_name;
+	cmd.parameters.session_rotation.uid = uid;
+	cmd.parameters.session_rotation.gid = gid;
+	cmd.parameters.session_rotation.trace_archive_chunk_id =
+			trace_archive_chunk_id;
+	cmd.parameters.session_rotation.location = location;
+
+	ret = run_command_wait(handle, &cmd);
+	if (ret) {
+		ret_code = LTTNG_ERR_UNK;
+		goto end;
+	}
+	ret_code = cmd.reply_code;
+end:
+	return ret_code;
+}
+
 void notification_thread_command_quit(
 		struct notification_thread_handle *handle)
 {
