@@ -22,6 +22,8 @@
 #include <lttng/lttng-error.h>
 #include <urcu/rculfhash.h>
 #include "notification-thread.h"
+#include "notification-thread-internal.h"
+#include "notification-thread-events.h"
 #include <common/waiter.h>
 
 struct notification_thread_data;
@@ -32,26 +34,11 @@ enum notification_thread_command_type {
 	NOTIFICATION_COMMAND_TYPE_UNREGISTER_TRIGGER,
 	NOTIFICATION_COMMAND_TYPE_ADD_CHANNEL,
 	NOTIFICATION_COMMAND_TYPE_REMOVE_CHANNEL,
+	NOTIFICATION_COMMAND_TYPE_SESSION_ROTATION_ONGOING,
+	NOTIFICATION_COMMAND_TYPE_SESSION_ROTATION_COMPLETED,
 	NOTIFICATION_COMMAND_TYPE_QUIT,
 };
 
-struct channel_key {
-	uint64_t key;
-	enum lttng_domain_type domain;
-};
-
-struct channel_info {
-	struct channel_key key;
-	char *session_name;
-	uid_t uid;
-	gid_t gid;
-	char *channel_name;
-	uint64_t capacity;
-	struct cds_lfht_node channels_ht_node;
-	/* call_rcu delayed reclaim. */
-	struct rcu_head rcu_node;
-};
-
 struct notification_thread_command {
 	struct cds_list_head cmd_list_node;
 
@@ -60,12 +47,31 @@ struct notification_thread_command {
 		/* Register/Unregister trigger. */
 		struct lttng_trigger *trigger;
 		/* Add channel. */
-		struct channel_info add_channel;
+		struct {
+			struct {
+				const char *name;
+				uid_t uid;
+				gid_t gid;
+			} session;
+			struct {
+				const char *name;
+				enum lttng_domain_type domain;
+				uint64_t key;
+				uint64_t capacity;
+			} channel;
+		} add_channel;
 		/* Remove channel. */
 		struct {
 			uint64_t key;
 			enum lttng_domain_type domain;
 		} remove_channel;
+		struct {
+			const char *session_name;
+			uid_t uid;
+			gid_t gid;
+			uint64_t trace_archive_chunk_id;
+			struct lttng_trace_archive_location *location;
+		} session_rotation;
 	} parameters;
 
 	/* lttng_waiter on which to wait for command reply (optional). */
@@ -83,7 +89,7 @@ enum lttng_error_code notification_thread_command_unregister_trigger(
 
 enum lttng_error_code notification_thread_command_add_channel(
 		struct notification_thread_handle *handle,
-		char *session_name, uid_t uid, gid_t gid,
+		char *session_name, uid_t session_uid, gid_t session_gid,
 		char *channel_name, uint64_t key,
 		enum lttng_domain_type domain, uint64_t capacity);
 
@@ -91,6 +97,18 @@ enum lttng_error_code notification_thread_command_remove_channel(
 		struct notification_thread_handle *handle,
 		uint64_t key, enum lttng_domain_type domain);
 
+enum lttng_error_code notification_thread_command_session_rotation_ongoing(
+		struct notification_thread_handle *handle,
+		const char *session_name, uid_t session_uid, gid_t session_gid,
+		uint64_t trace_archive_chunk_id);
+
+/* Ownership of location is transferred. */
+enum lttng_error_code notification_thread_command_session_rotation_completed(
+		struct notification_thread_handle *handle,
+		const char *session_name, uid_t session_uid, gid_t session_gid,
+		uint64_t trace_archive_chunk_id,
+		struct lttng_trace_archive_location *location);
+
 void notification_thread_command_quit(
 		struct notification_thread_handle *handle);
 
