@@ -385,7 +385,7 @@ void check_session_rotation_pending_on_consumers(struct ltt_session *session,
 				&exists_status);
 		if (ret) {
 			pthread_mutex_unlock(socket->lock);
-			ERR("Error occured while checking rotation status on consumer daemon");
+			ERR("Error occurred while checking rotation status on consumer daemon");
 			goto end;
 		}
 
@@ -414,7 +414,7 @@ skip_ust:
 				&exists_status);
 		if (ret) {
 			pthread_mutex_unlock(socket->lock);
-			ERR("Error occured while checking rotation status on consumer daemon");
+			ERR("Error occurred while checking rotation status on consumer daemon");
 			goto end;
 		}
 
@@ -529,38 +529,6 @@ int check_session_rotation_pending(struct ltt_session *session,
 		}
 	}
 
-	if (!session->active && !session->quiet_rotation) {
-		/*
-		 * A stop command was issued during the rotation, it is
-		 * up to the rotation completion check to perform the
-		 * renaming of the last chunk that was produced.
-		 */
-		ret = notification_thread_command_session_rotation_ongoing(
-				notification_thread_handle,
-				session->name,
-				session->uid,
-				session->gid,
-				session->most_recent_chunk_id.value);
-		if (ret != LTTNG_OK) {
-			ERR("[rotation-thread] Failed to notify notification thread of completed rotation for session %s",
-					session->name);
-		}
-
-		/* Ownership of location is transferred. */
-		location = session_get_trace_archive_location(session);
-		ret = notification_thread_command_session_rotation_completed(
-				notification_thread_handle,
-				session->name,
-				session->uid,
-				session->gid,
-				session->most_recent_chunk_id.value,
-				location);
-		if (ret != LTTNG_OK) {
-			ERR("[rotation-thread] Failed to notify notification thread of completed rotation for session %s",
-					session->name);
-		}
-	}
-
 	ret = 0;
 check_ongoing_rotation:
 	if (session->rotation_state == LTTNG_ROTATION_STATE_ONGOING) {
