@@ -83,6 +83,9 @@ DECL_COMMAND(track);
 DECL_COMMAND(untrack);
 DECL_COMMAND(metadata);
 DECL_COMMAND(regenerate);
+DECL_COMMAND(rotate);
+DECL_COMMAND(enable_rotation);
+DECL_COMMAND(disable_rotation);
 
 extern int cmd_help(int argc, const char **argv,
 		const struct cmd_struct commands[]);
