@@ -127,7 +127,7 @@ int cmd_save(int argc, const char **argv)
 {
 	int ret = CMD_SUCCESS, command_ret = CMD_SUCCESS, success;
 	int opt;
-	const char *session_name = NULL;
+	const char *session_name = NULL, *leftover = NULL;
 	poptContext pc;
 	struct lttng_save_session_attr *attr;
 
@@ -164,6 +164,13 @@ int cmd_save(int argc, const char **argv)
 		}
 	}
 
+	leftover = poptGetArg(pc);
+	if (leftover) {
+		ERR("Unknown argument: %s", leftover);
+		ret = CMD_ERROR;
+		goto end;
+	}
+
 	attr = lttng_save_session_attr_create();
 	if (!attr) {
 		ret = CMD_FATAL;
