@@ -355,6 +355,9 @@ ssize_t lttcomm_recvmsg_inet6_sock(struct lttcomm_sock *sock, void *buf,
 		len_last = iov[0].iov_len;
 		ret = recvmsg(sock->fd, &msg, flags);
 		if (ret > 0) {
+			if (flags & MSG_DONTWAIT) {
+				goto end;
+			}
 			iov[0].iov_base += ret;
 			iov[0].iov_len -= ret;
 			assert(ret <= len_last);
@@ -366,7 +369,7 @@ ssize_t lttcomm_recvmsg_inet6_sock(struct lttcomm_sock *sock, void *buf,
 		ret = len;
 	}
 	/* Else ret = 0 meaning an orderly shutdown. */
-
+end:
 	return ret;
 }
 
