@@ -31,10 +31,13 @@
 #include <lttng/save-internal.h>
 #include <lttng/channel-internal.h>
 #include <lttng/trigger/trigger-internal.h>
+#include <lttng/rotate-internal.h>
 #include <common/compat/socket.h>
 #include <common/uri.h>
 #include <common/defaults.h>
 #include <common/compat/uuid.h>
+#include <common/macros.h>
+#include <common/optional.h>
 
 #include <arpa/inet.h>
 #include <netinet/in.h>
@@ -57,49 +60,54 @@
 
 enum lttcomm_sessiond_command {
 	/* Tracer command */
-	LTTNG_ADD_CONTEXT                   = 0,
+	LTTNG_ADD_CONTEXT                     = 0,
 	/* LTTNG_CALIBRATE used to be here */
-	LTTNG_DISABLE_CHANNEL               = 2,
-	LTTNG_DISABLE_EVENT                 = 3,
-	LTTNG_LIST_SYSCALLS                 = 4,
-	LTTNG_ENABLE_CHANNEL                = 5,
-	LTTNG_ENABLE_EVENT                  = 6,
+	LTTNG_DISABLE_CHANNEL                 = 2,
+	LTTNG_DISABLE_EVENT                   = 3,
+	LTTNG_LIST_SYSCALLS                   = 4,
+	LTTNG_ENABLE_CHANNEL                  = 5,
+	LTTNG_ENABLE_EVENT                    = 6,
 	/* 7 */
 	/* Session daemon command */
-	LTTNG_CREATE_SESSION                = 8,
-	LTTNG_DESTROY_SESSION               = 9,
-	LTTNG_LIST_CHANNELS                 = 10,
-	LTTNG_LIST_DOMAINS                  = 11,
-	LTTNG_LIST_EVENTS                   = 12,
-	LTTNG_LIST_SESSIONS                 = 13,
-	LTTNG_LIST_TRACEPOINTS              = 14,
-	LTTNG_REGISTER_CONSUMER             = 15,
-	LTTNG_START_TRACE                   = 16,
-	LTTNG_STOP_TRACE                    = 17,
-	LTTNG_LIST_TRACEPOINT_FIELDS        = 18,
+	/* 8 */
+	LTTNG_DESTROY_SESSION                 = 9,
+	LTTNG_LIST_CHANNELS                   = 10,
+	LTTNG_LIST_DOMAINS                    = 11,
+	LTTNG_LIST_EVENTS                     = 12,
+	LTTNG_LIST_SESSIONS                   = 13,
+	LTTNG_LIST_TRACEPOINTS                = 14,
+	LTTNG_REGISTER_CONSUMER               = 15,
+	LTTNG_START_TRACE                     = 16,
+	LTTNG_STOP_TRACE                      = 17,
+	LTTNG_LIST_TRACEPOINT_FIELDS          = 18,
 
 	/* Consumer */
-	LTTNG_DISABLE_CONSUMER              = 19,
-	LTTNG_ENABLE_CONSUMER               = 20,
-	LTTNG_SET_CONSUMER_URI              = 21,
+	LTTNG_DISABLE_CONSUMER                = 19,
+	LTTNG_ENABLE_CONSUMER                 = 20,
+	LTTNG_SET_CONSUMER_URI                = 21,
 	/* 22 */
 	/* 23 */
-	LTTNG_DATA_PENDING                  = 24,
-	LTTNG_SNAPSHOT_ADD_OUTPUT           = 25,
-	LTTNG_SNAPSHOT_DEL_OUTPUT           = 26,
-	LTTNG_SNAPSHOT_LIST_OUTPUT          = 27,
-	LTTNG_SNAPSHOT_RECORD               = 28,
-	LTTNG_CREATE_SESSION_SNAPSHOT       = 29,
-	LTTNG_CREATE_SESSION_LIVE           = 30,
-	LTTNG_SAVE_SESSION                  = 31,
-	LTTNG_TRACK_PID                     = 32,
-	LTTNG_UNTRACK_PID                   = 33,
-	LTTNG_LIST_TRACKER_PIDS             = 34,
-	LTTNG_SET_SESSION_SHM_PATH          = 40,
-	LTTNG_REGENERATE_METADATA           = 41,
-	LTTNG_REGENERATE_STATEDUMP          = 42,
-	LTTNG_REGISTER_TRIGGER              = 43,
-	LTTNG_UNREGISTER_TRIGGER            = 44,
+	LTTNG_DATA_PENDING                    = 24,
+	LTTNG_SNAPSHOT_ADD_OUTPUT             = 25,
+	LTTNG_SNAPSHOT_DEL_OUTPUT             = 26,
+	LTTNG_SNAPSHOT_LIST_OUTPUT            = 27,
+	LTTNG_SNAPSHOT_RECORD                 = 28,
+	/* 29 */
+	/* 30 */
+	LTTNG_SAVE_SESSION                    = 31,
+	LTTNG_TRACK_PID                       = 32,
+	LTTNG_UNTRACK_PID                     = 33,
+	LTTNG_LIST_TRACKER_PIDS               = 34,
+	LTTNG_SET_SESSION_SHM_PATH            = 40,
+	LTTNG_REGENERATE_METADATA             = 41,
+	LTTNG_REGENERATE_STATEDUMP            = 42,
+	LTTNG_REGISTER_TRIGGER                = 43,
+	LTTNG_UNREGISTER_TRIGGER              = 44,
+	LTTNG_ROTATE_SESSION                  = 45,
+	LTTNG_ROTATION_GET_INFO               = 46,
+	LTTNG_ROTATION_SET_SCHEDULE           = 47,
+	LTTNG_SESSION_LIST_ROTATION_SCHEDULES = 48,
+	LTTNG_CREATE_SESSION_EXT              = 49
 };
 
 enum lttcomm_relayd_command {
@@ -123,6 +131,14 @@ enum lttcomm_relayd_command {
 	RELAYD_STREAMS_SENT                 = 16,
 	/* Ask the relay to reset the metadata trace file (2.8+) */
 	RELAYD_RESET_METADATA               = 17,
+	/* Ask the relay to rotate a set of stream files (2.11+) */
+	RELAYD_ROTATE_STREAMS                = 18,
+	/* Ask the relay to create a trace chunk (2.11+) */
+	RELAYD_CREATE_TRACE_CHUNK           = 19,
+	/* Ask the relay to close a trace chunk (2.11+) */
+	RELAYD_CLOSE_TRACE_CHUNK            = 20,
+	/* Ask the relay whether a trace chunk exists (2.11+) */
+	RELAYD_TRACE_CHUNK_EXISTS           = 21,
 };
 
 /*
@@ -130,7 +146,13 @@ enum lttcomm_relayd_command {
  */
 enum lttcomm_return_code {
 	LTTCOMM_CONSUMERD_SUCCESS            = 0,   /* Everything went fine. */
-	LTTCOMM_CONSUMERD_COMMAND_SOCK_READY = 1,   /* Command socket ready */
+	/*
+	 * Some code paths use -1 to express an error, others
+	 * negate this consumer return code. Starting codes at
+	 * 100 ensures there is no mix-up between this error value
+	 * and legitimate status codes.
+	 */
+	LTTCOMM_CONSUMERD_COMMAND_SOCK_READY = 100, /* Command socket ready */
 	LTTCOMM_CONSUMERD_SUCCESS_RECV_FD,          /* Success on receiving fds */
 	LTTCOMM_CONSUMERD_ERROR_RECV_FD,            /* Error on receiving fds */
 	LTTCOMM_CONSUMERD_ERROR_RECV_CMD,           /* Error on receiving command */
@@ -151,6 +173,14 @@ enum lttcomm_return_code {
 	LTTCOMM_CONSUMERD_CHANNEL_FAIL,             /* Channel creation failed. */
 	LTTCOMM_CONSUMERD_CHAN_NOT_FOUND,           /* Channel not found. */
 	LTTCOMM_CONSUMERD_ALREADY_SET,              /* Resource already set. */
+	LTTCOMM_CONSUMERD_ROTATION_FAIL,            /* Rotation has failed. */
+	LTTCOMM_CONSUMERD_SNAPSHOT_FAILED,          /* snapshot has failed. */
+	LTTCOMM_CONSUMERD_CREATE_TRACE_CHUNK_FAILED,/* Trace chunk creation failed. */
+	LTTCOMM_CONSUMERD_CLOSE_TRACE_CHUNK_FAILED, /* Trace chunk creation failed. */
+	LTTCOMM_CONSUMERD_INVALID_PARAMETERS,       /* Invalid parameters. */
+	LTTCOMM_CONSUMERD_TRACE_CHUNK_EXISTS_LOCAL, /* Trace chunk exists on consumer daemon. */
+	LTTCOMM_CONSUMERD_TRACE_CHUNK_EXISTS_REMOTE,/* Trace chunk exists on relay daemon. */
+	LTTCOMM_CONSUMERD_UNKNOWN_TRACE_CHUNK,      /* Unknown trace chunk. */
 
 	/* MUST be last element */
 	LTTCOMM_NR,						/* Last element */
@@ -247,13 +277,15 @@ struct lttcomm_session_msg {
 			uint32_t expression_len;
 			/* Length of following bytecode for filter. */
 			uint32_t bytecode_len;
-			/* exclusion data */
+			/* Exclusion count (fixed-size strings). */
 			uint32_t exclusion_count;
+			/* Userspace probe location size. */
+			uint32_t userspace_probe_location_len;
 			/*
 			 * After this structure, the following variable-length
 			 * items are transmitted:
 			 * - char exclusion_names[LTTNG_SYMBOL_NAME_LEN][exclusion_count]
-			 * - unsigned char filter_expression[expression_len]
+			 * - char filter_expression[expression_len]
 			 * - unsigned char filter_bytecode[bytecode_len]
 			 */
 		} LTTNG_PACKED enable;
@@ -321,10 +353,35 @@ struct lttcomm_session_msg {
 		struct {
 			uint32_t length;
 		} LTTNG_PACKED trigger;
+		struct {
+			uint64_t rotation_id;
+		} LTTNG_PACKED get_rotation_info;
+		struct {
+			/* enum lttng_rotation_schedule_type */
+			uint8_t type;
+			/*
+			 * If set == 1, set schedule to value, if set == 0,
+			 * clear this schedule type.
+			 */
+			uint8_t set;
+			uint64_t value;
+		} LTTNG_PACKED rotation_set_schedule;
+		struct {
+			/*
+			 * Includes the null-terminator.
+			 * Must be an absolute path.
+			 *
+			 * Size bounded by LTTNG_PATH_MAX.
+			 */
+			uint16_t home_dir_size;
+			uint64_t session_descriptor_size;
+			/* An lttng_session_descriptor follows. */
+		} LTTNG_PACKED create_session;
 	} u;
 } LTTNG_PACKED;
 
 #define LTTNG_FILTER_MAX_LEN	65536
+#define LTTNG_SESSION_DESCRIPTOR_MAX_LEN	65536
 
 /*
  * Filter bytecode data. The reloc table is located at the end of the
@@ -381,8 +438,21 @@ struct lttcomm_event_extended_header {
 	 * character.
 	 */
 	uint32_t nb_exclusions;
+
+	/*
+	 * Size of the event's userspace probe location (if applicable).
+	 */
+	uint32_t userspace_probe_location_len;
 } LTTNG_PACKED;
 
+/*
+ * Command header of the reply to an LTTNG_DESTROY_SESSION command.
+ */
+struct lttcomm_session_destroy_command_header {
+	/* enum lttng_session */
+	int32_t rotation_state;
+};
+
 /*
  * Data structure for the response from sessiond to the lttng client.
  */
@@ -404,14 +474,14 @@ struct lttcomm_lttng_output_id {
  * operation.
  */
 struct lttcomm_consumer_msg {
-	uint32_t cmd_type;	/* enum consumerd_command */
+	uint32_t cmd_type;	/* enum lttng_consumer_command */
 	union {
 		struct {
 			uint64_t channel_key;
 			uint64_t session_id;
+			/* ID of the session's current trace chunk. */
+			LTTNG_OPTIONAL_COMM(uint64_t) LTTNG_PACKED chunk_id;
 			char pathname[PATH_MAX];
-			uint32_t uid;
-			uint32_t gid;
 			uint64_t relayd_id;
 			/* nb_init_streams is the number of streams open initially. */
 			uint32_t nb_init_streams;
@@ -464,10 +534,15 @@ struct lttcomm_consumer_msg {
 			uint64_t session_id;			/* Tracing session id */
 			char pathname[PATH_MAX];		/* Channel file path. */
 			char name[LTTNG_SYMBOL_NAME_LEN];	/* Channel name. */
-			uint32_t uid;				/* User ID of the session */
-			uint32_t gid;				/* Group ID ot the session */
+			/* Credentials used to open the UST buffer shared mappings. */
+			struct {
+				uint32_t uid;
+				uint32_t gid;
+			} LTTNG_PACKED buffer_credentials;
 			uint64_t relayd_id;			/* Relayd id if apply. */
 			uint64_t key;				/* Unique channel key. */
+			/* ID of the session's current trace chunk. */
+			LTTNG_OPTIONAL_COMM(uint64_t) LTTNG_PACKED chunk_id;
 			unsigned char uuid[UUID_LEN];	/* uuid for ust tracer. */
 			uint32_t chan_id;			/* Channel ID on the tracer side. */
 			uint64_t tracefile_size;	/* bytes */
@@ -534,6 +609,57 @@ struct lttcomm_consumer_msg {
 		struct {
 			uint64_t session_id;
 		} LTTNG_PACKED regenerate_metadata;
+		struct {
+			uint32_t metadata; /* This is a metadata channel. */
+			uint64_t relayd_id; /* Relayd id if apply. */
+			uint64_t key;
+		} LTTNG_PACKED rotate_channel;
+		struct {
+			uint64_t session_id;
+			uint64_t chunk_id;
+		} LTTNG_PACKED check_rotation_pending_local;
+		struct {
+			uint64_t relayd_id;
+			uint64_t session_id;
+			uint64_t chunk_id;
+		} LTTNG_PACKED check_rotation_pending_relay;
+		struct {
+			/*
+			 * Relayd id, if applicable (remote).
+			 *
+			 * A directory file descriptor referring to the chunk's
+			 * output folder is transmitted if the chunk is local
+			 * (relayd_id unset).
+			 *
+			 * `override_name` is left NULL (all-zeroes) if the
+			 * chunk's name is not overridden.
+			 */
+			LTTNG_OPTIONAL_COMM(uint64_t) LTTNG_PACKED relayd_id;
+			char override_name[LTTNG_NAME_MAX];
+			uint64_t session_id;
+			uint64_t chunk_id;
+			uint64_t creation_timestamp;
+			LTTNG_OPTIONAL_COMM(struct {
+				uint32_t uid;
+				uint32_t gid;
+			} LTTNG_PACKED ) LTTNG_PACKED credentials;
+		} LTTNG_PACKED create_trace_chunk;
+		struct {
+			LTTNG_OPTIONAL_COMM(uint64_t) LTTNG_PACKED relayd_id;
+			uint64_t session_id;
+			uint64_t chunk_id;
+			uint64_t close_timestamp;
+			/* enum lttng_trace_chunk_command_type */
+			LTTNG_OPTIONAL_COMM(uint32_t) LTTNG_PACKED close_command;
+		} LTTNG_PACKED close_trace_chunk;
+		struct {
+			LTTNG_OPTIONAL_COMM(uint64_t) LTTNG_PACKED relayd_id;
+			uint64_t session_id;
+			uint64_t chunk_id;
+		} LTTNG_PACKED trace_chunk_exists;
+		struct {
+			lttng_uuid sessiond_uuid;
+		} LTTNG_PACKED init;
 	} u;
 } LTTNG_PACKED;
 
@@ -548,6 +674,10 @@ struct lttcomm_consumer_channel_monitor_msg {
 	 * Lowest and highest usage (bytes) at the moment the sample was taken.
 	 */
 	uint64_t lowest, highest;
+	/*
+	 * Sum of all the consumed positions for a channel.
+	 */
+	uint64_t total_consumed;
 } LTTNG_PACKED;
 
 /*
@@ -563,6 +693,12 @@ struct lttcomm_consumer_status_channel {
 	unsigned int stream_count;
 } LTTNG_PACKED;
 
+struct lttcomm_consumer_close_trace_chunk_reply {
+	enum lttcomm_return_code ret_code;
+	uint32_t path_length;
+	char path[];
+};
+
 #ifdef HAVE_LIBLTTNG_UST_CTL
 
 #include <lttng/ust-abi.h>
