@@ -1,4 +1,5 @@
 /*
+ * Copyright (C) 2015 - Michael Jeanson <mjeanson@efficios.com>
  * Copyright (C) 2013 - David Goulet <dgoulet@efficios.com>
  *
  * This program is free software; you can redistribute it and/or modify it
@@ -15,20 +16,29 @@
  * Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
+import java.io.IOException;
 import java.lang.Integer;
+import java.util.logging.Handler;
 import java.util.logging.Logger;
 import java.util.logging.Level;
 
-import org.lttng.ust.agent.LTTngAgent;
+import org.lttng.ust.agent.jul.LttngLogHandler;
 
-public class JTestLTTng
-{
-	private static LTTngAgent lttngAgent;
+public class JTestLTTng {
+
+	/**
+	 * Application start
+	 *
+	 * @param args
+	 *            Command-line arguments
+	 * @throws IOException
+	 * @throws InterruptedException
+	 */
+	public static void main(String args[]) throws IOException, InterruptedException {
 
-	public static void main(String args[]) throws Exception
-	{
 		Logger lttng = Logger.getLogger("JTestLTTng");
 		Logger lttng2 = Logger.getLogger("JTestLTTng2");
+
 		int nrIter = Integer.parseInt(args[0]);
 		int waitTime = Integer.parseInt(args[1]);
 		int fire_finest_tp = 0;
@@ -41,7 +51,11 @@ public class JTestLTTng
 			fire_second_tp = Integer.parseInt(args[3]);
 		}
 
-		lttngAgent = LTTngAgent.getLTTngAgent();
+		/* Instantiate a LTTngLogHandler object, and attach it to our loggers */
+		Handler lttngHandler = new LttngLogHandler();
+		lttng.addHandler(lttngHandler);
+		lttng2.addHandler(lttngHandler);
+
 		lttng.setLevel(Level.FINEST);
 
 		for (int iter = 0; iter < nrIter; iter++) {
@@ -57,6 +71,10 @@ public class JTestLTTng
 			lttng2.info("JUL second logger fired");
 		}
 
-		lttngAgent.dispose();
+		/*
+		 * Do not forget to close() all handlers so that the agent can shutdown
+		 * and the session daemon socket gets cleaned up explicitly.
+		 */
+		lttngHandler.close();
 	}
 }
